@@ -1,3 +1,4 @@
+
 from calendar import timegm
 import datetime as dt
 from itertools import islice
@@ -5,7 +6,7 @@ from collections import MutableSet, OrderedDict
 from threading import local
 from tzlocal import get_localzone
 from pytz import timezone, country_timezones, all_timezones, FixedOffset, utc, NonExistentTimeError, common_timezones
-
+from simpledate.fmt import strptime, reconstruct
 
 
 # A wrapper around the datetime, pytz and tzlocal packages.
@@ -16,6 +17,7 @@ from pytz import timezone, country_timezones, all_timezones, FixedOffset, utc, N
 
 
 # Build the various formats used by SimpleDateParser.
+from simpledate.utils import DebugLog, MRUSortedIterable, OrderedSet
 
 
 def add_timezone(*formats):
@@ -328,78 +330,6 @@ class SingleInstantTzError(SimpleDateError):
 
 
 
-# Useful base classes.
-
-
-class MRUSortedIterable:
-    '''
-    An iterable that re-orders the contents to move the most recently used item
-    (the last accessed on the previous iteration) to the start of the sequence.
-
-    IMPORTANT: Not thread safe.
-    '''
-
-    def __init__(self, data):
-        '''
-        :param data: The data to iterate over.
-        :return: An iterable that adapts to provide MRU values first.
-        '''
-        self._data = list(data)
-        self._i = 0
-
-    def __iter__(self):
-        '''
-        :return: A new iterator.
-        '''
-        # the iterator records the last accessed value in `_i`.  when a new
-        # iterable is requested that points to the last returned (and
-        # presumably used) value.  so at that point we can re-order the data.
-        if self._i:
-            self._data[0], self._data[1:self._i+1] = self._data[self._i], self._data[0:self._i]
-        for self._i, value in enumerate(self._data):
-            yield value
-            # reset on exhaustion.  usefully, this means that the final value was
-        # not OK, so another value was requested.  in this way we can avoid
-        # promoting the last value when no value was used.
-        self._i = 0
-
-
-class DebugLog:
-    '''
-    Base class supporting a simple log to stdout for debugging.  Is it possible
-    to use Python logging and do the same thing?
-    '''
-
-    def _get_log(self, debug):
-        '''
-        :param debug: True to enable logging.
-        :return: A logger that will print to stdout if `debug` is `True`
-        '''
-        return self._log if debug else self._drop
-
-    def _log(self, template, *args, **kargs):
-        '''
-        A logger that prints to stdout.
-
-        :param template: A string that can contain embedded {0}-style
-                         formatting.
-        :param args: Format arguments.
-        :param kargs: Named format arguments.
-        '''
-        print('%s: %s' % (self.__class__.__name__, template.format(*args, **kargs)))
-
-    def _drop(self, template, *args, **kargs):
-        '''
-        A null logger that discards its arguments.
-
-        :param template: Unused.
-        :param args: Unused.
-        :param kargs: Unused.
-        '''
-        pass
-
-
-
 # Classes implementing the core functionality.
 
 
@@ -553,7 +483,7 @@ class PyTzFactory(DebugLog):
         :param datetime: When the timezone is used.
         :param is_dst: Whether the timezone is daylight saving (used to resolve ambiguities during transition).
         :param country: A country code (or tuple of codes).  If given, only timezones in that country are considered.
-        :param unsafe: Do not check for multiple results.
+        :param unsafe: Take the first timezone found.
         :param debug: Print an explanation of the process followed to stdout?
         :return: A timezone consistent with the parameters given.
         '''
@@ -803,7 +733,7 @@ class SimpleDateParser(DebugLog):
                            `dt.tzinfo` instance.
         :param format: `None`, or a format to store in the final Date (`None`
                        will use the format that parsed the data).
-        :param unsafe: Do not check for multiple timezones.
+        :param unsafe: Take the first timezone found.
         :param debug: If true, print a description of the logic followed.
         :return: A SimpleDate instance, constructed from the given data.
         '''
@@ -813,7 +743,7 @@ class SimpleDateParser(DebugLog):
         for fmt in self._formats:
             try:
 
-                tt, fraction = _strptime(date, fmt)
+                tt, fraction, write_fmt = strptime(date, fmt)
                 log('Raw parse results for {0}: {1!r}, {2!r}', fmt, tt, fraction)
                 datetime = dt.datetime(*(tt[:6] + (fraction,)))
 
@@ -835,7 +765,7 @@ class SimpleDateParser(DebugLog):
 
                 datetime = tzinfo_localize(tzinfo, datetime, is_dst)
                 log('Parsed {0} with {1} to give {2} / {3}', date, fmt, datetime, tzinfo)
-                return SimpleDate(datetime=datetime, format=fmt if format is None else format, unsafe=unsafe, debug=debug)
+                return SimpleDate(datetime=datetime, format=write_fmt if (format is None or fmt is format) else format, unsafe=unsafe, debug=debug)
 
             except ValueError as e:
                 log('Failed to parse {0} with {1} ({2})', date, fmt, e)
@@ -999,53 +929,43 @@ class SimpleDate(DateTimeWrapper, DebugLog):
         For more flexible parsing of dates in strings, consider using
         `DateParser`.
 
-        TODO - wut?
-
-        A timestamp defines an exact time; for a timestamp the `tz_default`
-        parameter defines the 'local' part.
+        A timestamp defines an exact time; for a timestamp the `tz` parameter
+        functions as a conversion.
 
         All other values are defined relative to some timezone.  If this
-        is present in the value then it is used in the conversion to UTC and
-        stored in the Date instance for later re-conversion (eg when
-        displaying a string).  If a timezone is not part of the input value
-        then `tz_default`, or the locale value is `tz_default` is `None`,
-        is used.
+        is present in the value then it must be consistent with any given `tz`.
+        If no timezone is present in the value, and none is given, then
+        the local default is used.
 
         So, for example:
 
-          >>> SimpleDate(timestamp=0, tz_default='CLST')
-          1969-12-31 21:00:00.000000 CLST
+          >>> SimpleDate(timestamp=0, tz='CLST')
+          SimpleDate('1969-12-31 21:00:00.000000 CLST', tz='America/Santiago')
 
-        will store the UTC date for midnight, Jan 1 1970 (the Unix origin)
-        but convert that to the Chilean summer timezone for output (which, it
-        turns out, is 9pm the evening before).
+        will convert the UTC date for midnight, Jan 1 1970 (the Unix origin)
+        to Chilean summer time (which, it turns out, is 9pm the evening before).
 
-        And when a value contains a timezone, that timezone is saved for
-        output:
+        In contrast, for other inputs, the given value is taken as being
+        a literal value in that timezone:
 
-          >>> SimpleDate(dt.datetime(2012, 5, 19, 12, 0, tzinfo=timezone('US/Eastern')), tz_default='ignored')
-          2012-05-19 13:00:00.000000 EDT
-
-        (the shift from 12:00 to 13:00 is due to daylight saving being used
-        on that date).
+          >>> SimpleDate(dt.datetime(2012, 5, 19, 12, 0, tzinfo=timezone('US/Eastern')))
+          SimpleDate('2012-05-19 12:00:00.000000 EST', tz='US/Eastern')
 
         A string with timezone will be handled similarly:
 
-          >>> SimpleDate('2012-05-19 12:00 EDT', tz_default='ignored')
-          # TODO
+          >>> SimpleDate('2012-05-19 12:00 EDT')
+          SimpleDate('2012-05-19 12:00 EDT')
 
         But if the string did not have a timezone (ie just '2012-05-19 12:00')
         then it would have been assumed to have been in the timezone given:
 
-          >>> SimpleDate('2012-05-19 12:00', tz_default='CLT')
-          # TODO
+          >>> SimpleDate('2012-05-19 12:00', tz='CLT', format=DEFAULT_FORMAT)
+          SimpleDate('2012-05-19 12:00:00.000000 CLT')
 
         Similarly:
 
-          >>> SimpleDate(2012, 5, 19, 12, 0, tz_default='CLT')
-          # TODO
-
-        (except for the formatting, described next).
+          >>> SimpleDate(2012, 5, 19, 12, 0, tz='CLT')
+          SimpleDate('2012-05-19 12:00:00.000000 CLT')
 
         The `format` parameter describes the output format, used for display.
         If `None`, then an ISO-8601-like format is used, unless the value is
@@ -1073,7 +993,7 @@ class SimpleDate(DateTimeWrapper, DebugLog):
         :param format: The format used for output (also used to parse input string if `date_parser` is `None`).
         :param date_parser: Used to parse an input string (default DEFAULT_DATE_PARSER, combined with `format` if given).
         :param equality: How do we compare values?  See SimpleDate.EQUALITY.  Default is DEFAULT_EQUALITY unless `simple` given (in which case it is copied).
-        :param unsafe:
+        :param unsafe: Take the first timezone found.
         :param debug: If true, print a description of the logic followed.
         :return: A Date instance, containing the given data.
         '''
@@ -1298,6 +1218,13 @@ class SimpleDate(DateTimeWrapper, DebugLog):
 FACTORIES = local()
 
 def get_local(name, builder):
+    '''
+    Helper for thread local data.
+
+    :param name: The name of the attribute to get.
+    :param builder: A thunk to evaluate to generate the default value if missing.
+    :return: The current value of the attribute.
+    '''
     value = getattr(FACTORIES, name, None)
     if value is None:
         value = builder()
@@ -1305,6 +1232,13 @@ def get_local(name, builder):
     return value
 
 def best_guess_utc(date, debug=False):
+    '''
+    Try US timezones with US formats, then everything else.
+
+    :param date: A date to parse.
+    :param debug: If true, print a description of the logic followed.
+    :return: A UTC datetime.
+    '''
     us_date_parser = get_local('us_date_parser', lambda: SimpleDateParser(MONTH_FIRST + DEFAULT_FORMATS))
     eu_date_parser = get_local('eu_date_parser', lambda: SimpleDateParser(DAY_FIRST + DEFAULT_FORMATS))
     us_tz_factory = get_local('us_tz_factory', lambda: PyTzFactory(all_timezones, countries=['US']))
@@ -1316,93 +1250,6 @@ def best_guess_utc(date, debug=False):
     return date.utc.datetime
 
 
-# based on http://code.activestate.com/recipes/576694/
-
-
-class OrderedSet(MutableSet):
-
-    def __init__(self, iterable=None):
-        self.end = end = []
-        end += [None, end, end]         # sentinel node for doubly linked list
-        self.map = {}                   # key --> [key, prev, next]
-        if iterable is not None:
-            self |= iterable
-
-    def __len__(self):
-        return len(self.map)
-
-    def __contains__(self, key):
-        return key in self.map
-
-    def add(self, key):
-        if key not in self.map:
-            end = self.end
-            curr = end[1]
-            curr[2] = end[1] = self.map[key] = [key, curr, end]
-
-    def discard(self, key):
-        if key in self.map:
-            key, prev, next = self.map.pop(key)
-            prev[2] = next
-            next[1] = prev
-
-    def __iter__(self):
-        end = self.end
-        curr = end[2]
-        while curr is not end:
-            yield curr[0]
-            curr = curr[2]
-
-    def __reversed__(self):
-        end = self.end
-        curr = end[1]
-        while curr is not end:
-            yield curr[0]
-            curr = curr[1]
-
-    def pop(self, last=True):
-        if not self:
-            raise KeyError('set is empty')
-        key = self.end[1][0] if last else self.end[2][0]
-        self.discard(key)
-        return key
-
-    def __repr__(self):
-        if not self:
-            return '%s()' % (self.__class__.__name__,)
-        return '%s(%r)' % (self.__class__.__name__, list(self))
-
-    def __eq__(self, other):
-        if isinstance(other, OrderedSet):
-            return len(self) == len(other) and list(self) == list(other)
-        return set(self) == set(other)
-
-    # these methods added
-
-    @staticmethod
-    def union(*sets):
-        union = OrderedSet()
-        union.union(*sets)
-        return union
-
-    def union(self, *sets):
-        for set in sets:
-            self |= set
-
-    @staticmethod
-    def intersect(*sets):
-        intersection = OrderedSet()
-        if sets:
-            first = next(iter(sets))
-            for item in first:
-                rest = iter(sets)
-                next(rest)  # drop first
-                for set in rest:
-                    if item not in set:
-                        break
-                    intersection.add(item)
-        return intersection
-
 
 # the routines below are copied from _strptime.py, except that the
 # definition for timezone (Z) is changed from a list of specific options
@@ -1410,9 +1257,9 @@ class OrderedSet(MutableSet):
 
 try:
     from _thread import allocate_lock as _thread_allocate_lock
-except:
+except ImportError:
     from _dummy_thread import allocate_lock as _thread_allocate_lock
-from _strptime import TimeRE, _getlang, LocaleTime, _calc_julian_from_U_or_W
+from _strptime import _getlang, LocaleTime, _calc_julian_from_U_or_W
 from datetime import (date as datetime_date)
 import time
 from re import compile as re_compile
